@@ -1,17 +1,21 @@
 #!/usr/bin/python3
 
-import tempfile
-import sys
-import os
-import stat
 import argparse
-import getpass
-import pwd
-import subprocess
-import glob
-from subprocess import Popen, PIPE
-import importlib.machinery
 import errno
+import getpass
+import glob
+import importlib.machinery
+import os
+import pwd
+import stat
+import sys
+import subprocess
+import tempfile
+from subprocess import Popen, PIPE
+from typing import Optional
+
+BLUE = '\033[1;34m'
+BLACK = '\033[0m'
 
 for pgm in ('/usr/bin/editor', '/usr/bin/vim', '/usr/bin/nano', '/usr/bin/mcedit'):
     if os.path.isfile(pgm) and os.access(pgm, os.X_OK):
@@ -24,9 +28,11 @@ EDITOR = (os.environ.get('EDITOR') or
           os.environ.get('VISUAL') or
           editor)
 
-CRONTAB_DIR = '@statedir@'
+SELF = os.path.basename(sys.argv[0])
 
-SETGID_HELPER = '@libdir@/@package@/crontab_setgid'
+CRONTAB_DIR = '@statedir@'
+GENERATOR_DIR= '@generatordir@'
+SETGID_HELPER = '@libdir@/systemd-cron/crontab_setgid'
 
 HAS_SETGID =     os.geteuid() != 0 \
              and os.path.isfile(SETGID_HELPER) \
@@ -37,44 +43,8 @@ HAS_SETGID =     os.geteuid() != 0 \
 
 REBOOT_FILE = '/run/crond.reboot'
 
-args_parser = argparse.ArgumentParser(description='maintain crontab files for individual users')
 
-group = args_parser.add_mutually_exclusive_group()
-
-args_parser.add_argument('-u', '--user', type=str, dest='user', default=getpass.getuser(),
-        help='''It specifies the name of the user whose crontab is to be
- tweaked. If this option is not given, crontab examines "your" crontab, i.e., the crontab of the person
- executing the command. Note that su(8) can confuse crontab and that if you are running inside of su(8) you
- should always use the -u option for safety's sake. The first form of this command is used to install a new
- crontab from some named file or standard input if the pseudo-filename "-" is given.''')
-
-args_parser.add_argument('file', type=str, default='-', nargs='?')
-
-group.add_argument('-l', '--list', dest='action', action='store_const', const='list',
-        help='''The current crontab will be displayed on standard output.''')
-
-group.add_argument('-r', '--remove', dest='action', action='store_const', const='remove',
-        help='''The current crontab will be removed.''')
-
-group.add_argument('-e', '--edit', dest='action', action='store_const', const='edit',
-        help='''This option is used to edit the current crontab using the editor
- specified by the VISUAL or EDITOR environment variables. After
- you exit from the editor, the modified crontab will be installed
- automatically.''')
-
-group.add_argument('-s', '--show', dest='action', action='store_const', const='show',
-        help='''Show all user who have a crontab.''')
-
-args_parser.add_argument('-i', '--ask', dest='ask', action='store_true', default=False,
-        help='''This option modifies the -r option to prompt the user for a
- 'y/Y' response before actually removing the crontab.''')
-
-#args_parser.add_argument('-s', '--secure', dest='secure', action='store_true', default=False,
-        #help='''It will append the current SELinux security context string as an
- #MLS_LEVEL setting to the crontab file before editing / replacement occurs
- #- see the documentation of MLS_LEVEL in crontab(5).''')
-
-def confirm(message):
+def confirm(message:str) -> bool:
     while True:
         answer = input(message).lower()
         if answer not in 'yn':
@@ -83,36 +53,81 @@ def confirm(message):
 
         return answer == 'y'
 
-def check(cron_file):
+
+def blue(line:str) -> None:
+    if sys.stdin.isatty():
+        print('%s%s%s' % (BLUE, line, BLACK))
+    else:
+        print(line)
+
+
+def translate(line:str, args) -> None:
+    loader = importlib.machinery.SourceFileLoader('name',
+                os.path.join(GENERATOR_DIR, 'systemd-crontab-generator'))
+    parser = loader.load_module()
+    line = args.file
+    print(line)
+
+    job = parser.Job('-', line)
+    job.parse_crontab_auto()
+    job.decode()
+    job.decode_command()
+    job.generate_schedule()
+
+    blue('# /run/systemd/generator/<unit>.timer')
+    print(job.generate_timer())
+    print('#Persistent=true')
+    print()
+
+    blue('# /run/systemd/generator/<unit>.service')
+    print(job.generate_service())
+
+
+def check(cron_file:str) -> bool:
     good = True
+    loader = importlib.machinery.SourceFileLoader('name',
+                os.path.join(GENERATOR_DIR, 'systemd-crontab-generator'))
+    parser = loader.load_module()
     for job in parser.parse_crontab(cron_file, withuser=False):
-        if 'c' not in job:
+        if not job.valid:
             good = False
-            sys.stderr.write('%s: truncated line in %s: %s\n' % (SELF, cron_file, job['l']))
-        elif 'p' in job:
-            if job['p'] not in ['reboot', 'minutely', 'hourly', 'daily', 'midnight', 'weekly',
+            sys.stderr.write('%s: truncated line in %s: %s\n' % (SELF, cron_file, job.line))
+        elif type(job.period) is str:
+            if job.period not in ['reboot', 'minutely', 'hourly', 'daily', 'midnight', 'weekly',
                                 'monthly', 'quarterly',
                                 'semi-annually', 'semiannually', 'bi-annually', 'biannually',
                                 'annually', 'yearly']:
                 good = False
-                sys.stderr.write("%s: unknown schedule in %s: %s\n" % (SELF, cron_file, job['l']))
-        elif 0 in job['M'] or 0 in job['d']:
-                good = False
-                sys.stderr.write("%s: month and day can't be 0 in %s: %s\n" % (SELF, cron_file, job['l']))
-        else:
-            if not len(job['M']) or not len(job['d']) or not len(job['h']) or not len(job['m']):
+                sys.stderr.write("%s: unknown schedule in %s: %s\n" % (SELF, cron_file, job.line))
+        elif '0' in job.timespec_month or '0' in job.timespec_day:
                 good = False
+                sys.stderr.write("%s: month and day can't be 0 in %s: %s\n" % (SELF, cron_file, job.line))
     return good
 
-def try_chmod(cron_file, user):
-    if CRON_GROUP:
+
+def try_chmod(cron_file:Optional[str]=None,
+              user:Optional[str]=None) -> None:
+    '''trying to fixup things if run as root'''
+    try:
+        CRON_GROUP = os.stat(SETGID_HELPER).st_gid
+    except:
+        return
+
+    try:
+        os.chown(CRONTAB_DIR, 0, CRON_GROUP)
+        os.chmod(CRONTAB_DIR, stat.S_ISVTX | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | stat.S_IWGRP | stat.S_IXGRP)
+    except:
+        pass
+
+    if cron_file and user:
         try:
             os.chown(cron_file, pwd.getpwnam(user).pw_uid, CRON_GROUP)
             os.chmod(cron_file, stat.S_IRUSR | stat.S_IWUSR)
         except (PermissionError, KeyError):
             pass
 
-def list(cron_file, args):
+
+def list(cron_file:str, args) -> None:
     try:
         with open(cron_file, 'r') as f:
             sys.stdout.write(f.read())
@@ -133,7 +148,9 @@ def list(cron_file, args):
         else:
             raise
 
-def remove(cron_file, args):
+
+def remove(cron_file:str, args):
+    try_chmod()
     if not args.ask or confirm('Are you sure you want to delete %s (y/n)? ' % cron_file):
         try:
             os.unlink(cron_file)
@@ -155,7 +172,8 @@ def remove(cron_file, args):
             else:
                 raise
 
-def edit(cron_file, args):
+
+def edit(cron_file:str, args):
     tmp = tempfile.NamedTemporaryFile(mode='w+', encoding='UTF-8', delete=False, prefix='crontab_')
 
     try:
@@ -224,7 +242,8 @@ def edit(cron_file, args):
             sys.stderr.write("unexpected error, your edit is kept here:%s\n" % tmp.name)
             raise
 
-def show(cron_file, args):
+
+def show(cron_file:str, args) -> None:
     if os.geteuid() != 0:
         sys.exit("must be privileged to use -s")
 
@@ -236,7 +255,8 @@ def show(cron_file, args):
         except KeyError:
             sys.stderr.write("WARNING: crontab found with no matching user: %s\n" % user)
 
-def replace(cron_file, args):
+
+def replace(cron_file:str, args) -> None:
     if args.file == '-':
         try:
             crontab = sys.stdin.read()
@@ -287,8 +307,41 @@ def replace(cron_file, args):
             raise
 
 
-if __name__ == '__main__':
-    SELF = os.path.basename(sys.argv[0])
+def main() -> None:
+    args_parser = argparse.ArgumentParser(description='maintain crontab files for individual users')
+
+    group = args_parser.add_mutually_exclusive_group()
+
+    args_parser.add_argument('-u', '--user', type=str, dest='user', default=getpass.getuser(),
+            help='''It specifies the name of the user whose crontab is to be
+     tweaked. If this option is not given, crontab examines "your" crontab, i.e., the crontab of the person
+     executing the command. Note that su(8) can confuse crontab and that if you are running inside of su(8) you
+     should always use the -u option for safety's sake. The first form of this command is used to install a new
+     crontab from some named file or standard input if the pseudo-filename "-" is given.''')
+
+    args_parser.add_argument('file', type=str, default='-', nargs='?')
+
+    group.add_argument('-l', '--list', dest='action', action='store_const', const='list',
+            help='''The current crontab will be displayed on standard output.''')
+
+    group.add_argument('-r', '--remove', dest='action', action='store_const', const='remove',
+           help='''The current crontab will be removed.''')
+
+    group.add_argument('-e', '--edit', dest='action', action='store_const', const='edit',
+           help='''This option is used to edit the current crontab using the editor
+     specified by the VISUAL or EDITOR environment variables. After
+     you exit from the editor, the modified crontab will be installed
+     automatically.''')
+
+    group.add_argument('-s', '--show', dest='action', action='store_const', const='show',
+            help='''Show all user who have a crontab.''')
+
+    args_parser.add_argument('-i', '--ask', dest='ask', action='store_true', default=False,
+            help='''This option modifies the -r option to prompt the user for a
+     'y/Y' response before actually removing the crontab.''')
+
+    group.add_argument('-t', '--translate', dest='action', action='store_const', const='translate',
+            help='''Translate one crontab line and print the result.''')
 
     # try to fixup CRONTAB_DIR if it has not been handled in package script
     try:
@@ -297,18 +350,6 @@ if __name__ == '__main__':
     except:
         sys.exit("%s doesn't exists!" % CRONTAB_DIR)
 
-    try:
-        CRON_GROUP = os.stat(SETGID_HELPER).st_gid
-    except:
-        CRON_GROUP = None
-
-    if CRON_GROUP:
-        try:
-            os.chown(CRONTAB_DIR, 0, CRON_GROUP)
-            os.chmod(CRONTAB_DIR, stat.S_ISVTX | stat.S_IRUSR | stat.S_IWUSR | stat.S_IXUSR | stat.S_IWGRP | stat.S_IXGRP)
-        except:
-            pass
-
     args = args_parser.parse_args()
 
     # deluser in adduser package expect this behaviour
@@ -332,9 +373,10 @@ if __name__ == '__main__':
             'edit': edit,
             'remove': remove,
             'show': show,
+            'translate': translate,
             }.get(args.action, replace)
 
-    loader = importlib.machinery.SourceFileLoader('name', '@generatordir@/systemd-crontab-generator')
-    parser = loader.load_module()
-
     action(cron_file, args)
+
+if __name__ == '__main__':
+    main()
